/****************************************************************************

  slp.c : $B6!5kE@G[CVLdBj(B

  Programmed by Yuusuke Abe  2000-11-22

****************************************************************************/


#include <stdio.h>		/* NULL, fputs(), printf() */
#include <stdarg.h>		/* va_list, va_start(), va_end(), vfprintf() */
#include <stdlib.h>		/* EXIT_..., exit(), malloc(), free() */
#include <math.h>		/* fabs() */
#include <float.h>		/* DBL_MAX */
#include "heap.h"
#include "graph.h"
#include "clist.h"


/* $B%(%i!<$rI=<($7%W%m%0%i%`$r=*N;(B */
void error(char *s, ...)
{
    va_list args;

    va_start(args, s);
    vfprintf(stderr, s, args);
    va_end(args);
    fputs("\n", stderr);
    exit(EXIT_FAILURE);
}


/* $B=LLs5!G=IU$-%0%i%U:n@.(B */
void make_graph_c(GRAPH * g, GRAPH_C * gc)
{
    int i;
    double temp;
    EDGE *e;
    RANGE *r;
    LAMINAR *lam, *lam_g;
    LAM_NUM *ln, *ln_g, **cn, *lns;

    /* $B;^>pJs$N%3%T!<(B */
    for (i = 0; i < g->n; i++) {
	e = g->adjlist[i];
	while (e != NULL) {
	    if (e->node > i)
		insert_edge_c(i, e->node, e->weight, gc);
	    e = e->next;
	}
    }

    /* $B%l%s%8>pJs$N%3%T!<(B */
    for (i = 1; i < g->n; i++) {
	r = (RANGE *) malloc(sizeof(RANGE));
	r->value = g->range[i]->value;
	gc->range[i] = r;
    }

    /* LAMINAR$B>pJs$N%3%T!<(B */
    lam_g = g->laminar;
    lam = (LAMINAR *) malloc(sizeof(LAMINAR));
    gc->laminar = lam;
    do {
	lam->k[0] = lam_g->k[0];	/* LAMINAR$B$NHO0O$r%3%T!<(B */
	lam->k[1] = lam_g->k[1];
	lam->k[2] = lam_g->k[2];
	ln_g = lam_g->num;
	ln = (LAM_NUM *) malloc(sizeof(LAM_NUM));
	lam->num = ln;
	do {
	    ln->number = ln_g->number;
	    ln_g = ln_g->next;
	    if (ln_g != NULL) {
		ln->next = (LAM_NUM *) malloc(sizeof(LAM_NUM));
		ln = ln->next;
	    }
	} while (ln_g != NULL);
	lam_g = lam_g->next;
	if (lam_g != NULL) {
	    lam->next = (LAMINAR *) malloc(sizeof(LAMINAR));
	    lam = lam->next;
	}
    } while (lam_g != NULL);

    /* $B@aE@$N=LLs>uBVJ];}9=B$BN$N=i4|2=(B */
    for (i = 0; i < g->n; i++) {
	lns = (LAM_NUM *) malloc(sizeof(LAM_NUM));
	lns->number = i;
	lns->next = NULL;
	gc->lamnum[i] = lns;
    }

    /* LAMINAR$BLZ$N%3%T!<(B($B8=:_%]%$%s%?6&DL(B) */
    gc->tree = g->tree;

}


/* $B:GBgNY@\=g=x(B(Maximum Adjacency Ordering)$B$r<hF@(B
** i : $B7W;;3+;O@aE@HV9f(B
** gc : $B=LLs5!G=IU$-%0%i%U9=B$BN$X$N%]%$%s%?(B
*/
void get_mao(int i, GRAPH_C * gc)
{
    int j;
    EDGE_C *e;
    HEAP h;

    /* i $B0J30$N@aE@$K$h$k%R!<%W:n@.(B */
    create_heap(gc->n, &h);
    for (j = 0; j < gc->n; j++) {
	if (j == i || gc->state[j] == DELETED)
	    continue;
	insert_heap(j, 0, &h);
    }

    gc->otn[0] = i;
    gc->nto[i] = 0;

    /* $B:GBgNY@\=g=x7W;;(B */
    for (j = 1; h.n >= 1; j++) {
	e = gc->adjlist[i];
	while (e != NULL) {
	    /* $B=E$_J,$@$1%-!<$rA}2C(B */
	    if (member(e->node, &h))
		increase_key(e->node, e->weight, &h);
	    e = e->next;
	}
	gc->otn[j] = i = delete_max(&h);
	gc->nto[i] = j;
    }

    /* $B%R!<%W3+J|(B */
    free_heap(&h);
}


/* $B:G>.%+%C%H<hF@(B
** gc : $B=LLs5!G=IU$-%0%i%U9=B$BN$X$N%]%$%s%?(B
** $BLa$jCM(B : $B:G>.%+%C%H(B
*/
double get_mincut(GRAPH_C * gc)
{
    int i;
    double mincut, d;
    EDGE_C *e;

    mincut = DBL_MAX;
    for (i = 0; i < gc->n - 1; i++) {
	get_mao(0, gc);

	d = 0;			/* $B%+%C%H$NCM(B */
	e = gc->adjlist[gc->otn[gc->ne - 1]];
	while (e != NULL) {
	    d += e->weight;
	    e = e->next;
	}
	if (mincut > d)
	    mincut = d;

	contract_node_c(gc->otn[gc->ne - 2], gc->otn[gc->ne - 1], gc);
    }

    return mincut;
}


/* $B:GBgNY@\=g=x$NKvHx$N#2@aE@$,:G>.%+%C%H$K$J$k$^$G=LLs(B */
void find_mincut_pair(double purpose, GRAPH_C * gc, CLIST * cl)
{
    int s, t;
    double d;
    EDGE_C *e;

    do {
	get_mao(0, gc);

	d = 0;			/* $B%+%C%H$NCM(B */
	e = gc->adjlist[gc->otn[gc->ne - 1]];
	while (e != NULL) {
	    d += e->weight;
	    e = e->next;
	}
	if (d == purpose)
	    break;

	s = gc->otn[gc->ne - 2];
	t = gc->otn[gc->ne - 1];
	contract_node_c(s, t, gc);
	contract_clist(s, t, cl);
    } while (gc->ne > 2);
}

/*
LAMINAR$BLZ$r99?7$9$k(B
s : $B=LLs8e:o=|$5$l$kE@(B
t : $B=LLs8e;D$kE@(B
*/
void renewal_tree(int s, int t, TREE ** tree, double *flag)
{
    TREE *new;

    new = (TREE *) malloc(sizeof(TREE));
    new->vernum = 0;
    new->right = tree[s];
    new->left = tree[t];
    new->parent = tree[0];
    new->left->parent = new;
    new->right->parent = new;
    tree[s] = NULL;
    tree[t] = new;

    if (flag != NULL) {
	new->Vstate = LAM;
	new->area[0] = flag[0];
	new->area[1] = flag[1];
    } else {
	new->Vstate = DUMMY;
    }

}

double calc_k2(double cut, RANGE * r, double purpose)
{

    double k, aj;
    int nor;
    RANGE *r_now, *r_next;

    r_now = r;
    r_next = r->next;
    nor = 1;

    do {
	aj += r_next->value;
	k = (aj + (r_now->value - cut)) / nor;
	if ((k > purpose) && (r->next != NULL)
	    && (r_next->next->value > k)) {
	    r_next = r_next->next;
	    nor++;
	    continue;
	} else
	    break;
    } while (1);
    return k;
}


/* $B%l%s%8$H%l%s%8$ND9$5$N99?7(B
** t  : $B=LLs8e$K;D$C$?@\E@HV9f(B
** gc : $B=LLs5!G=IU$-%0%i%U$X$N%]%$%s%?(B
*/
double *renewal_range(int t, GRAPH_C * gc, double purpose)
{

    int i, nor;
    double cut, aj, k, weight, *flag;
    EDGE_C *e1, *e2;
    RANGE *r;
    LAMINAR *lam, *lamt;
    LAM_NUM *ln, *cn1, *cn2;


    /* $B=LLs$7$?@\E@$H(Bs$B$H$N4V$N;^$N=E$_$N=i4|2=(B */
    e1 = gc->adjlist[0];
    e2 = gc->adjlist[t];
    for (i = 0; i < gc->n; i++) {
	if (e1->node == t) {
	    e1->weight = 0.0;
	    break;
	} else
	    e1 = e1->next;
    }
    for (i = 0; i < gc->n; i++) {
	if (e2->node == 0) {
	    e2->weight = 0.0;
	    break;
	} else
	    e2 = e2->next;
    }

    /* $B=LLs$7$?@\E@<~$j$N%+%C%H$r7W;;(B */
    e2 = gc->adjlist[t];
    cut = 0.0;
    do {
	if (e2 != NULL) {
	    cut += e2->weight;
	    e2 = e2->next;
	} else
	    e2 = e2->next;
    } while (e2 != NULL);

    /* $B%l%s%8CM$N99?7(B */
    r = gc->range[t];
    aj = 0.0;
    nor = 1;
    if (r->value > cut) {
	k = calc_k2(cut, r, purpose);
    } else if (r->value <= cut) {
	k = 0.0;
    }

    /* $B%l%s%8$NCM$,%+%C%H$NCM$h$j>.$5$$$H$-$O(B
       k=0.0$B$H$$$&CM$K$9$k(B */
    r = gc->range[t];
    lamt = gc->laminar;
    if (k == 0.0) {
    } else if (k <= purpose) {

	/* LAMINAR$B$K$3$N$H$-$NCM$rJ]B8$9$k(B[cut, k] */
	lam = (LAMINAR *) malloc(sizeof(LAMINAR));
	cn1 = (LAM_NUM *) malloc(sizeof(LAM_NUM));
	lam->num = cn1;
	cn1->next = NULL;
	cn2 = gc->lamnum[t];
	do {
	    if (cn2 != NULL) {
		cn1->number = cn2->number;
		cn2 = cn2->next;
		if (cn2 == NULL) {
		    break;
		}
		cn1->next = (LAM_NUM *) malloc(sizeof(LAM_NUM));
		cn1 = cn1->next;
		cn1->next = NULL;
	    }
	} while (1);

	lam->k[0] = cut;
	lam->k[1] = k;
	lam->k[2] = '\0';
	lam->next = NULL;
	do {
	    if (lamt->next == NULL) {
		lamt->next = lam;
		break;
	    } else
		lamt = lamt->next;
	} while (1);

	/* LAMINAR$B$K4^$^$l$k@aE@HV9f$r%3%T!<(B */

	r->value = cut;
	r = r->next;
	do {
	    if (r->value >= k) {
		break;
	    } else if (r->value < k) {
		r->value = k;
		r = r->next;
	    }
	} while (r != NULL);
    } else if (k > purpose) {
	r->value = cut;
    }

    /* $B=LLs$7$?@aE@$H(Bs$B$H$N4V$K?7$?$J=E$_(B($BD9$5(B)$B$rD%$k(B */
    if (cut > gc->range[t]->value) {
	weight = purpose - gc->range[t]->value;
    } else
	weight = purpose - cut;

    e1 = gc->adjlist[0];
    e2 = gc->adjlist[t];
    for (i = 0; i < gc->n; i++) {
	if (e1->node == t) {
	    e1->weight = weight;
	    break;
	} else
	    e1 = e1->next;
    }
    for (i = 0; i < gc->n; i++) {
	if (e2->node == 0) {
	    e2->weight = weight;
	    break;
	} else
	    e2 = e2->next;
    }

    /* LAMINAR$B$H$J$C$?$+$I$&$+$N%U%i%0(B */
    if (k == 0.0) {
	flag = NULL;
    } else {
	flag = (double *) calloc(2, sizeof(double));
	flag[0] = cut;
	flag[1] = k;
    }
    return flag;
}


/* $B=LLs8e$N%l%s%8$N99?7(B
** s : $B:o=|$5$l$k@\E@HV9f(B 
** t : $B=LLs8e;D$k@\E@HV9f(B
*/
int contract_range_c(int s, int t, GRAPH_C * gc, double purpose, int inc)
{

    int i;
    double temp, *flag;
    RANGE *r, *r2;
    LAM_NUM *ln;

    r = gc->range[t];
    ln = gc->lamnum[t];

    /* $B%l%s%8$N=LLs(B */
    do {
	if (r->next == NULL) {
	    r->next = gc->range[s];
	    gc->range[s] = NULL;
	    break;
	} else
	    r = r->next;
    } while (1);

    /* $B=LLs>uBV$N99?7(B */
    do {
	if (ln->next == NULL) {
	    ln->next = gc->lamnum[s];
	    gc->lamnum[s] = NULL;
	    break;
	} else
	    ln = ln->next;
    } while (1);

    /* $B%l%s%8$N%=!<%F%#%s%0(B ver.$B%P%V%j%#(B */
    for (i = 1; i < gc->n; i++) {
	if (gc->state[i] == EXIST) {
	    for (r = gc->range[i]; r != NULL; r = r->next) {
		for (r2 = r->next; r2 != NULL; r2 = r2->next) {
		    if (r->value > r2->value) {
			temp = r->value;
			r->value = r2->value;
			r2->value = temp;
		    }
		}
	    }
	}
    }

    /* $B%l%s%8$H%l%s%8$ND9$5$N99?7(B */
    flag = renewal_range(t, gc, purpose);

    /* LAMINAR$BLZ$N99?7(B */
    renewal_tree(s, t, gc->tree, flag);

    return 0;

}


/* $B=LLs4X?t(B */
int rec_contract(double purpose, GRAPH_C * gc, int *aw, int inc)
{

    int s, t, temp, i;

    get_mao(0, gc);


    s = gc->otn[gc->ne - 2];
    t = gc->otn[gc->ne - 1];

    /* $BBg$-$$HV9f$ND:E@$r;D$9(B */
    if (s > t) {
	temp = s;
	s = t;
	t = temp;
    }

    /* $B;^$N=LLs(B */
    contract_node_c(s, t, gc);

    /* $B%l%s%8$N=LLs(B */
    contract_range_c(s, t, gc, purpose, inc);

    return 0;
}

/* $B%l%s%8$r99?7$9$k:]$NogCM$r7W;;(B */
double calc_k(int nor, double cut, RANGE * r, double purpose, double aj,
	      double a1)
{

    double k;

    aj += r->value;
    k = (aj - (cut - a1)) / nor;

    if ((k > purpose) && (r->next != NULL)) {
	k = calc_k(++nor, cut, r->next, purpose, aj, a1);
	return k;
    }

    return k;

}

/* $B%U%m!<7W;;MQ%0%i%U:n@.(B
** gc : $B=LLs5!G=IU$-%0%i%U9=B$BN$X$N%]%$%s%?(B
** gf : $B%U%m!<7W;;MQ%0%i%U$X$N%]%$%s%?(B
*/
void make_graph_f(GRAPH_C * gc, GRAPH_F * gf)
{
    int i, node;
    EDGE_C *e;

    for (i = 0; i < gc->ne - 1; i++) {
	node = gc->otn[i];
	e = gc->adjlist[node];
	while (e != NULL) {
	    if (gc->nto[e->node] > i)
		insert_directed_edge_f(e->node, node, e->weight, gf);
	    e = e->next;
	}
    }
}


/* $B%U%m!<7W;;(B
** mincut : $B:G>.%+%C%H(B
** gc :
** gf : $B%U%m!<7W;;MQ%0%i%U$X$N%]%$%s%?(B
*/
void get_flow(double mincut, GRAPH_C * gc, GRAPH_F * gf)
{
    int i, s, t, node;
    double f, *fbuf;
    EDGE_F *e;

    /* $B@aE@$KN.$l$k%U%m!<$rJ];}$9$kG[Ns$r3NJ](B */
    fbuf = (double *) malloc(gf->n * sizeof(double));
    for (i = 0; i < gf->n; i++)
	fbuf[i] = 0;

    /* $BI=<((B */
    s = gc->otn[gc->ne - 1];
    t = gf->adjlist[s]->node;
    printf("flow (%d ==> %d : %f):\n", s, t, mincut);

    /* $B%U%m!<7W;;(B */
    fbuf[s] = +mincut;
    fbuf[t] = -mincut;
    for (i = gc->ne - 1; i > 0; i--) {
	node = gc->otn[i];
	e = gf->adjlist[node];
	f = fbuf[node];
	if (f == 0)
	    continue;

	while (e != NULL) {
	    /* $B;^$N=E$_$,%U%m!<$HEy$7$$$+$h$jBg$-$$>l9g(B */
	    if (e->weight >= fabs(f)) {
		if (f > 0) {
		    e->flow = f;
		    printf("%d ==> %d : %f (weight=%f)\n", node, e->node,
			   f, e->weight);
		} else {
		    e->flow = f;
		    printf("%d <== %d : %f (weight=%f)\n", node, e->node,
			   -f, e->weight);

		    fbuf[e->node] += f;
		    break;
		}

		/* $B;^$N=E$_$,%U%m!<$h$j$b>.$5$$>l9g(B */
		if (f > 0) {
		    e->flow = e->weight;
		    fbuf[e->node] += e->weight;
		    f -= e->weight;
		    printf("%d ==> %d : %f (weight=%f)\n",
			   node, e->node, e->weight, e->weight);
		} else {
		    e->flow = -e->weight;
		    fbuf[e->node] += -e->weight;
		    f -= -e->weight;
		    printf("%d <== %d : %f (weight=%f)\n",
			   node, e->node, e->weight, e->weight);
		}
		e = e->next;
	    }
	}

	free(fbuf);
    }
}

/* $B%U%m!<$,:GBg$^$GN.$l$F$$$k;^$r=|5n$7(B, $BN>J}8~$K;^$,$"$k@aE@$r=LLs$7$?%0%i%U(B
** $B$r<hF@$9$k(B
** gc :
** gf : $B%U%m!<7W;;MQ%0%i%U$X$N%]%$%s%?(B
** gd : $B=LLs5!G=IU$-M-8~%0%i%U$X$N%]%$%s%?(B
** cl : $B=LLs%j%9%H9=B$BN$X$N%]%$%s%?(B
*/
void get_contracted_graph(GRAPH_C * gc, GRAPH_F * gf, GRAPH_D * gd,
			  CLIST * cl)
{
    int i, node, nn, *nbuf;
    EDGE_F *e;

    nbuf = (int *) malloc(gf->n * sizeof(int));

    /* $B=LLs$:$_@aE@$r:o=|(B */
    for (i = 0; i < gd->n; i++) {
	if (is_contracted(i, cl))
	    delete_node_d(i, gd);
    }

    /* $B;^A^F~(B */
    for (i = gc->ne - 1; i > 0; i--) {
	node = gc->otn[i];
	e = gf->adjlist[node];
	while (e != NULL) {
	    if (e->flow == e->weight)
		insert_directed_edge_d(e->node, node, e->weight, gd);
	    else if (e->flow == -e->weight)
		insert_directed_edge_d(node, e->node, e->weight, gd);
	    else if (!is_contracted(node, cl))
		contract_clist(node, e->node, cl);
	    e = e->next;
	}
    }

    /* $B=LLs(B */
    for (i = gc->ne - 1; i >= 0; i--) {
	nn = get_member(gc->otn[i], nbuf, cl);
	if (nn <= 0)
	    continue;
	contract_nodes_d(nn, nbuf, gd);
    }

    free(nbuf);
}


int index;


void depth_first_search(int i, int *visit, int *order, GRAPH_D * gd)
{
    EDGE_D *e;

    visit[i] = 1;
    e = gd->adjlist[i];
    while (e != NULL) {
	if (visit[e->node] == 0)
	    depth_first_search(e->node, visit, order, gd);
	e = e->next;
    }
    order[index] = i;
    index++;
}


void topological_sort(int s, int order[], GRAPH_D * gd)
{
    int *visit;

    visit = (int *) calloc(gd->n, sizeof(int));

    index = 0;
    depth_first_search(s, visit, order, gd);

    free(visit);
}

/* LAMINAR$BLZ$r:n@.(B */
void make_tree_g(TREE ** tree, int n, double *cut)
{
    TREE *temp;

    tree[n] = (TREE *) malloc(sizeof(TREE));
    temp = tree[n];

    temp->right = NULL;
    temp->left = NULL;
    temp->parent = tree[0];
    temp->vernum = n;
    temp->Vstate = LAM;
    temp->area[0] = cut[n];
    temp->area[1] = -1;
}


/* $BL\I8CM$X$NITB-J,$rIU2C(B */
int add_weight(int n, GRAPH * g, int *aw)
{
    int roop1;
    double *cut, purpose = 0.0;
    EDGE *e;
    RANGE *r;
    LAMINAR *lam;
    LAM_NUM *ln;

    cut = (double *) malloc(n * sizeof(double));

    /* $BA4$F$N6I=j%+%C%H$NCM$N7W;;(B */
    for (roop1 = 1; roop1 < n; roop1++) {
	e = g->adjlist[roop1];
	cut[roop1] = 0;
	do {
	    cut[roop1] += e->weight;
	    e = e->next;
	} while (e != NULL);
	/* $B6I=j%+%C%H$N:GBgCM$r<hF@(B */
	if (purpose < cut[roop1]) {
	    purpose = cut[roop1];
	}
    }

    purpose = (purpose * 2) + 1;	/* $BL\I8CM(BK$B$r@_Dj(B */

    lam = (LAMINAR *) malloc(sizeof(LAMINAR));
    ln = (LAM_NUM *) malloc(sizeof(LAM_NUM));
    lam->next = NULL;
    ln->next = NULL;
    g->laminar = lam;

    /* $B6I=j%+%C%H$NCM$,L\I8CM(BK$B$KB-$j$J$$>l9gIU$1B-$9(B */
    for (roop1 = 1; roop1 < n; roop1++) {
	aw[roop1] += purpose - cut[roop1];

	r = (RANGE *) malloc(sizeof(RANGE));	/* $B%l%s%8MQ9=B$BN$NNN0h3NJ](B */
	r->value = cut[roop1];	/* $B%l%s%8$NCM$r$$$l$k(B($B=i4|CM(B) */
	r->next = NULL;		/* $B%l%s%8$O:G=i0l$D$J$N$G$=$l0J9_$r(BNULL$B$K$9$k(B */
	g->range[roop1] = r;
	insert_edge(0, roop1, purpose - cut[roop1], g);	/* $B%l%s%8$ND9$5$r(B0$B$H$N;^$N=E$_$H$9$k(B */

	lam->k[0] = cut[roop1];
	lam->k[1] = 0;
	lam->k[2] = '\0';

	ln->number = roop1;
	lam->num = ln;

	/* $BLZ$r:n$k(B */
	make_tree_g(g->tree, roop1, cut);

	if (roop1 < n - 1) {
	    lam->next = (LAMINAR *) malloc(sizeof(LAMINAR));
	    ln = (LAM_NUM *) malloc(sizeof(LAM_NUM));
	    lam = lam->next;
	    lam->next = NULL;
	    ln->next = NULL;
	}

    }
    return purpose;
}

/* $BL\I8CM$X$NITB-J,$rIU2C(B */
int add_weight_c(int p, int n, GRAPH_C * g, int *aw)
{
    int cut, roop1;
    EDGE_C *e;

    /* $B6I=j%+%C%H$NCM$N7W;;(B */
    for (roop1 = 1; roop1 < n; roop1++) {
	e = g->adjlist[roop1];
	cut = 0;
	while (e != NULL) {
	    cut += e->weight;
	    e = e->next;
	}
	e = g->adjlist[roop1];

	/* $B6I=j%+%C%H$NCM$,L\I8CM$KB-$j$J$$>l9gIU$1B-$9(B */
	if ((p - cut > 0) && (g->state[roop1] != DELETED)) {
	    for (;;) {
		if (e->node == 0) {
		    e->weight += (double) (p - cut);
		    break;
		}
		e = e->next;
	    }
	    e = g->adjlist[0];
	    for (;;) {
		if (e->node == roop1) {
		    e->weight += (double) (p - cut);
		    break;
		}
		e = e->next;
	    }
	    aw[roop1] += p - cut;
	}
	printf("cut%2d : %2d      not enough : %2d     aw : %2d\n", roop1,
	       cut, (int) (p - cut), aw[roop1]);
    }
    printf("\n");
    return 0;
}

/* LAMINAR$BI=<(4X?t(B */
void print_laminar(LAMINAR * lam)
{

    LAMINAR *l;
    LAM_NUM *l_n;

    l = lam;

    /* LAMINAR$B$NI=<((B */
    for (; l != NULL;) {
	l_n = l->num;
	printf("LAMINAR : size[%2d, %2d]      vertex : [", l->k[0],
	       l->k[1]);
	do {
	    printf("%2d ", l_n->number);
	    l_n = l_n->next;
	} while (l_n != NULL);
	printf("]\n");
	l = l->next;
    }

}

/* LAMINAR$BLZ$N=PNO4X?t(B */
void print_lamtree(TREE * tree)
{
    if (tree->Vstate == LAM) {
	printf("(");
    }
    if (tree->right != NULL) {
	print_lamtree(tree->right);
    } else if (tree->right == NULL) {
	printf("%2d [%2d, %2d]", tree->vernum, (int) tree->area[0],
	       (int) tree->area[1]);
	if (tree->Vstate == LAM) {
	    printf(")");
	}
	return;
    }
    if (tree->left != NULL) {
	print_lamtree(tree->left);
    } else if (tree->left == NULL) {
	printf("%2d [%2d, %2d]", tree->vernum, (int) tree->area[0],
	       (int) tree->area[1]);
	if (tree->Vstate == LAM) {
	    printf(")");
	}
	return;
    }
    if (tree->Vstate == LAM) {
	printf(" [%2d, %2d])", (int) tree->area[0], (int) tree->area[1]);
    }

}

/* LAMINAR$BLZ$+$i%l%s%8$NCM$rCj=P$9$k4X?t(B */
void add_range(HT * ht, TREE * tree)
{

    RANGE *ran;

    if (tree == NULL) {
	return;
    }

    if (tree->right != NULL) {
	add_range(ht, tree->right);
    }

    if (tree->left != NULL) {
	add_range(ht, tree->left);
    }

    if ((tree->left == NULL) || (tree->Vstate == LAM)) {
	ht->tail->value = tree->area[0];
	ht->tail->next = (RANGE *) calloc(1, sizeof(RANGE));
	ht->tail = ht->tail->next;
	if (tree->area[1] != -1) {
	    ht->tail->value = tree->area[1];
	    ht->tail->next = (RANGE *) calloc(1, sizeof(RANGE));
	    ht->tail = ht->tail->next;
	}
    }
    return;
}

int sp;
int sp_count;

/* $B6!5kE@>l=j=PNO4X?t(B */
void PrintSupplyPoint(TREE * tree)
{

    if (tree->right != NULL) {
	PrintSupplyPoint(tree->right);
    } else if (tree->right == NULL) {
	printf(" %2d,", tree->vernum);
    }

}

/* $B6!5kE@$N;;=P4X?t(B */
void CulcSupplyPoint(TREE * tree, double range, double ran_next)
{

    int count_temp = -2;

    if (tree->Vstate == LAM) {
	if (((tree->area[0] <= range) && (tree->area[1] >= ran_next))
	    || ((tree->area[0] <= range) && (tree->area[1] == -1))) {
	    sp_count++;
	    count_temp = sp_count;
	}
    }

    if (tree->right != NULL) {
	CulcSupplyPoint(tree->right, range, ran_next);
    }
    if (tree->left != NULL) {
	CulcSupplyPoint(tree->left, range, ran_next);
    }

    if (tree->Vstate == LAM) {
	if (((tree->area[0] <= range) && (tree->area[1] >= ran_next))
	    || ((tree->area[0] <= range) && (tree->area[1] == -1))) {
	    sp_count++;
	}
	if (sp_count == (count_temp + 1)) {
	    /* $B6!5kE@>l=j$N=PNO4X?t(B */
	    PrintSupplyPoint(tree);
	    sp++;
	}
    }

}

/* LAMINAR$BLZ$+$iCj=P$5$l$?%l%s%8$NCM$r@0M}$9$k4X?t(B */
void set_range(HT * ht)
{

    RANGE *range, *range1;
    double temp;

    /* $B%j%9%H$N:G8e$NM>7W$J%9%Z!<%9$r%+%C%H(B */
    for (range = ht->top; range != NULL; range = range->next) {
	if (range->next == ht->tail) {
	    range->next = NULL;
	    ht->tail = range;
	}
    }

    /* $B%j%9%H$N%=!<%F%#%s%0(B */
    for (range = ht->top; range->next != NULL; range = range->next) {
	for (range1 = range->next; range1 != NULL; range1 = range1->next) {
	    if (range->value > range1->value) {
		temp = range->value;
		range->value = range1->value;
		range1->value = temp;
	    }
	}
    }

    /* $B$*$J$8?tCM$r%j%9%H$+$i:o=|(B */
    for (range = ht->top; range->next != NULL; range = range->next) {
	for (range1 = range->next; range1 != NULL; range1 = range1->next) {
	    if (range->value == range1->value) {
		range->next = range1->next;
	    }
	}
	if (range->next == NULL)
	    break;
    }

}

/* $B6!5kE@$N?t$rI=<($9$k4X?t(B */
void print_sp(TREE ** tree, int num)
{

    int i = 0;
    RANGE *range;
    HT *ht;

    ht = (HT *) calloc(1, sizeof(HT));
    range = (RANGE *) calloc(1, sizeof(RANGE));
    ht->top = range;
    ht->tail = range;

    for (i = 1; i < num; i++) {
	add_range(ht, tree[i]);
    }

    set_range(ht);

    printf("\nNumber of SupplyPoint : \n");
    /* $BFCDj$N%l%s%8$NHO0O$KBP$9$k6!5kE@$N?t$N=PNO(B */
    printf("area [  0, %3.2lf] --->  ( 1 )    1 points\n", ht->top->value);
    for (range = ht->top; range->next != NULL; range = range->next) {
	sp_count = 0;
	sp = 0;
	printf("area [ %3.2lf, %3.2lf] ---> (", range->value,
	       range->next->value);
	for (i = 1; i < num; i++) {
	    if (tree[i] != NULL) {
		CulcSupplyPoint(tree[i], range->value, range->next->value);
	    }
	}
	printf(")    %2d points\n", sp);
    }
    printf("area [ %3.2lf, $B!g(B] ---> %2dpoints\n", range->value,
	   num - 1);

}

/* $B>r7oIU$-(B($BFCDjL\I8CM$KBP$9$k(B)LAMINAR$B$NI=<((B CertainLaminar*/
void print_Clam(int k, LAMINAR * lam)
{

    LAM_NUM *ln;

    do {
	if (lam != NULL) {
	    ln = lam->num;
	    if (lam->k[0] < k) {
		if ((lam->k[1] >= k) || (lam->k[1] == 0)) {
		    if (lam->k[1] > 0) {
			printf("Range : [%3d, %3d]     vertex[", lam->k[0],
			       lam->k[1]);
		    } else if (lam->k[1] == 0) {
			printf("Range : [%3d, +$B!g(B]     vertex[",
			       lam->k[0]);
		    }
		    do {
			if (ln != NULL) {
			    printf("%2d ", ln->number);
			    ln = ln->next;
			} else
			    break;
		    } while (1);
		    printf("]\n");
		}
	    }
	    lam = lam->next;
	} else if (lam == NULL) {
	    break;
	}
    } while (1);

}

/* $B%a%$%s4X?t(B */
int main(int argc, char *argv[])
{
    GRAPH g;
    GRAPH_F gf;
    GRAPH_C gc;
    GRAPH_D gd;
    CLIST cl;
    RANGE *r;
    LAMINAR *l;

    int i, s, t, done, *order, *aw, num, k;
    double mincut, purpose;

    /* $B%0%i%UF~NO(B */
    done = input_graph(&g);
    if (done != 0)
	error("input error");

    num = g.n;
    aw = (int *) malloc(g.n * sizeof(int));

    purpose = add_weight(num, &g, aw);

    /* $B=LLs5!G=IU$-%0%i%U:n@.(B */
    create_graph_c(num, &gc);
    make_graph_c(&g, &gc);

    /* $B=LLs5!G=IU$-%0%i%U:F@_Dj(B */
    init_graph_c(&gc);
    make_graph_c(&g, &gc);

    /* $B>r7o$K9gCW$9$k$^$G=LLs(B */
    for (i = 0; i < g.n - 3; i++) {
	rec_contract(purpose, &gc, aw, i);
    }

    /* $B6!5kE@$N?t$r=PNO$9$k(B */
    print_sp(gc.tree, num);
    printf("\n");

    return EXIT_SUCCESS;
}
